@@ -99,6 +99,12 @@ enum {
   COLS_MAX
 };
 
+static void
+tree_selection_changed (GtkTreeSelection *selection, GtkDialog *dialog)
+{
+	gtk_dialog_set_response_sensitive (dialog, GTK_RESPONSE_ACCEPT, gtk_tree_selection_get_selected (selection, NULL, NULL));
+}
+
 /* Callback for ProcessNetworkProfile. If we have more than one username,
  we need to let the user select. */
 static uint32_t
@@ -112,7 +118,7 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
 	GtkWidget *dialog, *view;
-	GtkVBox *vbox;
+	GtkBox *content_area;
 	const gchar *username = (const gchar *)data;
 
 	/* If we can find the exact username, then find & return its index. */
@@ -133,8 +139,8 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 	/*TODO : Fix strings*/
 	dialog = gtk_dialog_new_with_buttons (_("Select username"),
 					      NULL, GTK_DIALOG_MODAL,
-					      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+					      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 					      NULL);
 
 	/*Tree View */
@@ -149,6 +155,9 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 						     -1, _("Username"), renderer,
 						     "text", COL_MAPI_ACCOUNT, NULL);
 
+	gtk_tree_view_column_set_resizable (gtk_tree_view_get_column (GTK_TREE_VIEW (view), 0), TRUE);
+	gtk_tree_view_column_set_resizable (gtk_tree_view_get_column (GTK_TREE_VIEW (view), 1), TRUE);
+
 	/* Model for TreeView */
 	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (store));
@@ -173,17 +182,24 @@ create_profile_callback (struct SRowSet *rowset, gpointer data)
 	}
 
 	/* Pack the TreeView into dialog's content area */
-	vbox = (GtkVBox *)gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_box_pack_start (GTK_BOX (vbox), view, TRUE, TRUE, 6);
-	gtk_widget_show_all (GTK_WIDGET (vbox));
+	content_area = GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog)));
+
+	gtk_box_pack_start (content_area, gtk_label_new (_("There are more users with similar user name on a server.\nPlease select that you would like to use from the below list.")), TRUE, TRUE, 6);
+	gtk_box_pack_start (content_area, view, TRUE, TRUE, 6);
+
+	gtk_widget_show_all (GTK_WIDGET (content_area));
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+	g_signal_connect (selection, "changed", G_CALLBACK (tree_selection_changed), dialog);
+	tree_selection_changed (selection, GTK_DIALOG (dialog));
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (response == GTK_RESPONSE_ACCEPT) {
 	       /* Get the index from the selected value */
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-		gtk_tree_selection_get_selected (selection, NULL, &iter);
-		gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX,
-				    &index, -1);
+		if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+			gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COL_MAPI_INDEX, &index, -1);
+		else
+			index = rowset->cRows + 1;
 	} else /* If we return a value > available, we are canceling the login.*/
 	       index = rowset->cRows + 1;
 
@@ -349,7 +365,7 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 {
 	EMConfigTargetAccount *target_account;
 	CamelURL *url;
-	GtkWidget *hbox = NULL;
+	GtkWidget *hgrid = NULL;
 	gint row;
 
 	target_account = (EMConfigTargetAccount *)data->config->target;
@@ -370,7 +386,7 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 		g_object_get (data->parent, "n-rows", &row, NULL);
 
 		/* Domain name & Authenticate Button */
-		hbox = gtk_hbox_new (FALSE, 6);
+		hgrid = g_object_new (GTK_TYPE_GRID, "column-homogeneous", FALSE, "column-spacing", 6, "orientation", GTK_ORIENTATION_HORIZONTAL, NULL);
 		label = gtk_label_new_with_mnemonic (_("_Domain name:"));
 		gtk_widget_show (label);
 
@@ -378,16 +394,16 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), domain_name);
 		if (domain_value && *domain_value)
 			gtk_entry_set_text (GTK_ENTRY (domain_name), domain_value);
-		gtk_box_pack_start (GTK_BOX (hbox), domain_name, FALSE, FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (hgrid), domain_name);
 		g_signal_connect (domain_name, "changed", G_CALLBACK(domain_entry_changed), data->config);
 
 		auth_button = gtk_button_new_with_mnemonic (_("_Authenticate"));
-		gtk_box_pack_start (GTK_BOX (hbox), auth_button, FALSE, FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (hgrid), auth_button);
 		g_signal_connect (auth_button, "clicked",  G_CALLBACK (validate_credentials), data->config);
 
 		gtk_table_attach (GTK_TABLE (data->parent), label, 0, 1, row, row+1, 0, 0, 0, 0);
-		gtk_widget_show_all (GTK_WIDGET (hbox));
-		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (hbox), 1, 2, row, row+1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
+		gtk_widget_show_all (GTK_WIDGET (hgrid));
+		gtk_table_attach (GTK_TABLE (data->parent), GTK_WIDGET (hgrid), 1, 2, row, row+1, GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
 		row++;
 
@@ -399,7 +415,7 @@ org_gnome_exchange_mapi_account_setup (EPlugin *epl, EConfigHookItemFactoryData
 	}
 
 	camel_url_free (url);
-	return hbox;
+	return hgrid;
 }
 
 gboolean
@@ -616,7 +632,7 @@ exchange_mapi_cursor_change (GtkTreeView *treeview, ESource *source)
 static GtkWidget *
 exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType folder_type)
 {
-	GtkWidget *vbox, *label, *scroll, *tv;
+	GtkWidget *vgrid, *label, *scroll, *tv;
 	gchar *uri_text, *profile = NULL;
 	ESourceGroup *group;
 	gint row;
@@ -652,19 +668,19 @@ exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType
 	if (conn)
 		g_object_unref (conn);
 
-	vbox = gtk_vbox_new (FALSE, 6);
+	vgrid = g_object_new (GTK_TYPE_GRID, "column-homogeneous", FALSE, "column-spacing", 6, "orientation", GTK_ORIENTATION_VERTICAL, NULL);
 
 	if (folder_type == MAPI_FOLDER_TYPE_CONTACT) {
-		gtk_container_add (GTK_CONTAINER (parent), vbox);
+		gtk_container_add (GTK_CONTAINER (parent), vgrid);
 	} else {
 		g_object_get (parent, "n-rows", &row, NULL);
-		gtk_table_attach (GTK_TABLE (parent), vbox, 0, 2, row+1, row+2, GTK_FILL|GTK_EXPAND, 0, 0, 0);
+		gtk_table_attach (GTK_TABLE (parent), vgrid, 0, 2, row+1, row+2, GTK_FILL|GTK_EXPAND, 0, 0, 0);
 	}
 
 	label = gtk_label_new_with_mnemonic (_("_Location:"));
 	gtk_widget_show (label);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (vgrid), label);
 
 	rcell = gtk_cell_renderer_text_new ();
 	tvc = gtk_tree_view_column_new_with_attributes (acc, rcell, "text", NAME_COL, NULL);
@@ -687,10 +703,10 @@ exchange_mapi_create (GtkWidget *parent, ESource *source, ExchangeMAPIFolderType
 	g_signal_connect (G_OBJECT (tv), "cursor-changed", G_CALLBACK (exchange_mapi_cursor_change), source);
 	gtk_widget_show_all (scroll);
 
-	gtk_box_pack_start (GTK_BOX (vbox), scroll, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (vgrid), scroll);
 
-	gtk_widget_show_all (vbox);
-	return vbox;
+	gtk_widget_show_all (vgrid);
+	return vgrid;
 }
 
 GtkWidget *
@@ -708,13 +724,13 @@ exchange_mapi_create_calendar (EPlugin *epl, EConfigHookItemFactoryData *data)
 	ExchangeMAPIFolderType folder_type;
 
 	switch (t->source_type) {
-	case E_CAL_SOURCE_TYPE_EVENT:
+	case E_CAL_CLIENT_SOURCE_TYPE_EVENTS:
 		folder_type = MAPI_FOLDER_TYPE_APPOINTMENT;
 		break;
-	case E_CAL_SOURCE_TYPE_TODO:
+	case E_CAL_CLIENT_SOURCE_TYPE_TASKS:
 		folder_type = MAPI_FOLDER_TYPE_TASK;
 		break;
-	case E_CAL_SOURCE_TYPE_JOURNAL:
+	case E_CAL_CLIENT_SOURCE_TYPE_MEMOS:
 		folder_type = MAPI_FOLDER_TYPE_MEMO;
 		break;
 	default:
@@ -861,13 +877,13 @@ exchange_mapi_cal_commit (EPlugin *epl, EConfigTarget *target)
 	g_free (uri_text);
 
 	switch (t->source_type) {
-		case E_CAL_SOURCE_TYPE_EVENT:
+		case E_CAL_CLIENT_SOURCE_TYPE_EVENTS:
 			type = olFolderCalendar;
 			break;
-		case E_CAL_SOURCE_TYPE_TODO:
+		case E_CAL_CLIENT_SOURCE_TYPE_TASKS:
 			type = olFolderTasks;
 			break;
-		case E_CAL_SOURCE_TYPE_JOURNAL:
+		case E_CAL_CLIENT_SOURCE_TYPE_MEMOS:
 			type = olFolderNotes;
 			break;
 		default:
