@@ -59,7 +59,6 @@ struct _CamelMapiStorePrivate {
 	ExchangeMapiConnection *conn;
 
 	gchar *base_url;
-	gchar *storage_path;
 
 	GHashTable *id_hash; /*get names from ids*/
 	GHashTable *name_hash;/*get ids from names*/
@@ -74,9 +73,7 @@ struct _CamelMapiStorePrivate {
 G_DEFINE_TYPE (CamelMapiStore, camel_mapi_store, CAMEL_TYPE_OFFLINE_STORE)
 
 /* service methods */
-static gboolean	mapi_construct(CamelService *, CamelSession *,
-				     CamelProvider *, CamelURL *,
-				     GError **);
+static void mapi_store_constructed (GObject *object);
 static gchar	*mapi_get_name(CamelService *, gboolean );
 static gboolean	mapi_connect_sync(CamelService *, GCancellable *cancellable, GError **);
 static gboolean	mapi_disconnect_sync(CamelService *, gboolean , GCancellable *cancellable, GError **);
@@ -256,7 +253,9 @@ mapi_get_folder_with_type (CamelStore *store, guint folder_type, GCancellable *c
 }
 
 static CamelFolderInfo *
-mapi_convert_to_folder_info (CamelMapiStore *store, ExchangeMAPIFolder *folder, const gchar *url, GError **error)
+mapi_convert_to_folder_info (CamelMapiStore *store,
+                             ExchangeMAPIFolder *folder,
+                             GError **error)
 {
 	gchar *name;
 	gchar *parent, *id = NULL;
@@ -313,17 +312,15 @@ mapi_convert_to_folder_info (CamelMapiStore *store, ExchangeMAPIFolder *folder,
 	mapi_id_folder = exchange_mapi_folder_get_parent_id (folder);
 	parent = g_strdup_printf ("%016" G_GINT64_MODIFIER "X", mapi_id_folder);
 
-	fi->name = name;
+	fi->display_name = name;
 
 	par_name = mapi_folders_hash_table_name_lookup (store, parent, TRUE);
 	if (par_name != NULL) {
 		gchar *str = g_strconcat (par_name, "/", name, NULL);
 
 		fi->full_name = str; /* takes ownership of the string */
-		fi->uri = g_strconcat (url, str, NULL);
 	} else {
 		fi->full_name = g_strdup (name);
-		fi->uri = g_strconcat (url, "", name, NULL);
 	}
 
 	/*name_hash returns the container id given the name */
@@ -361,13 +358,12 @@ remove_path_from_store_summary (const gchar *path, gpointer value, CamelMapiStor
 		fi = camel_folder_info_new ();
 		fi->unread = -1;
 		fi->total = -1;
-		fi->uri = g_strdup (camel_store_info_uri (mstore->summary, si));
-		fi->name = g_strdup (camel_store_info_name (mstore->summary, si));
+		fi->display_name = g_strdup (camel_store_info_name (mstore->summary, si));
 		fi->full_name = g_strdup (camel_mapi_store_info_full_name (mstore->summary, si));
-		if (!fi->name && fi->full_name) {
-			fi->name = strrchr (fi->full_name, '/');
-			if (fi->name)
-				fi->name = g_strdup (fi->name + 1);
+		if (!fi->display_name && fi->full_name) {
+			fi->display_name = strrchr (fi->full_name, '/');
+			if (fi->display_name)
+				fi->display_name = g_strdup (fi->display_name + 1);
 		}
 
 		camel_store_folder_unsubscribed (CAMEL_STORE (mstore), fi);
@@ -386,7 +382,6 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	CamelMapiStorePrivate  *priv = store->priv;
 	gboolean status;
 	GSList *folder_list = NULL, *temp_list = NULL, *list = NULL;
-	gchar *url, *temp_url;
 	gboolean subscription_list = FALSE;
 	CamelFolderInfo *info = NULL;
 	CamelMapiStoreInfo *mapi_si = NULL;
@@ -445,16 +440,6 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	temp_list = folder_list;
 	list = folder_list;
 
-	url = camel_url_to_string (CAMEL_SERVICE(store)->url,
-				   (CAMEL_URL_HIDE_PASSWORD|
-				    CAMEL_URL_HIDE_PARAMS|
-				    CAMEL_URL_HIDE_AUTH));
-	if ( url[strlen(url) - 1] != '/') {
-		temp_url = g_strconcat (url, "/", NULL);
-		g_free ((gchar *)url);
-		url = temp_url;
-	}
-
 	/*populate the hash table for finding the mapping from container id <-> folder name*/
 	for (;temp_list != NULL; temp_list = g_slist_next (temp_list) ) {
 		const gchar *full_name = NULL;
@@ -478,7 +463,8 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 				full_name = tmp;
 			}
 		} else {
-			tmp = escape_slash (full_name);
+			/* known full_name - everything is escaped already */
+			tmp = g_strdup (full_name);
 			full_name = tmp;
 		}
 
@@ -505,7 +491,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 			continue;
 
 		if ( folder->container_class == MAPI_FOLDER_TYPE_MAIL) {
-			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, NULL);
+			info = mapi_convert_to_folder_info (store, folder, NULL);
 			info->flags |= CAMEL_MAPI_FOLDER_MAIL;
 			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
 
@@ -539,7 +525,7 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 			camel_folder_info_free (info);
 		} else if (folder->category == MAPI_FAVOURITE_FOLDER) {
 			gchar *fid, *pfid = NULL;
-			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, NULL);
+			info = mapi_convert_to_folder_info (store, folder, NULL);
 			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
 			fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
 						exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(folder_list->data)));
@@ -566,8 +552,6 @@ mapi_folders_sync (CamelMapiStore *store, guint32 flags, GError **error)
 	camel_store_summary_touch ((CamelStoreSummary *)store->summary);
 	camel_store_summary_save ((CamelStoreSummary *)store->summary);
 
-	g_free (url);
-
 	g_slist_foreach (list, (GFunc) exchange_mapi_folder_free, NULL);
 	g_slist_free (list);
 
@@ -629,12 +613,12 @@ static void
 fix_folder_names (CamelFolderInfo *fi)
 {
 	while (fi) {
-		if (fi->name && strchr (fi->name, '\\')) {
+		if (fi->display_name && strchr (fi->display_name, '\\')) {
 			gchar *unescaped;
 
-			unescaped = unescape_slash (fi->name);
-			g_free (fi->name);
-			fi->name = unescaped;
+			unescaped = unescape_slash (fi->display_name);
+			g_free (fi->display_name);
+			fi->display_name = unescaped;
 		}
 
 		if (fi->child)
@@ -653,7 +637,7 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 	GPtrArray *folders;
 	gchar *path, *name;
 	gint i;
-	gboolean subscribed, favourites = FALSE, subscription_list = FALSE;
+	gboolean subscribed, subscription_list = FALSE;
 
 	subscription_list = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST);
 	subscribed = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED);
@@ -709,20 +693,9 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 				folder_name = strrchr(store_info_path,'/');
 				if (folder_name != NULL)
 					store_info_path = ++folder_name;
-
-				favourites = TRUE;
 			}
 
 			fi = mapi_build_folder_info(mapi_store, parent_name, store_info_path);
-			if (favourites) {
-				CamelURL *url;
-				url = camel_url_new(mapi_store->priv->base_url,NULL);
-				url->path = g_strdup_printf("/%s", camel_store_info_path((CamelStoreSummary *)mapi_store->summary, si));
-				g_free (fi->uri);
-				fi->uri = camel_url_to_string(url,CAMEL_URL_HIDE_ALL);
-				camel_url_free (url);
-			}
-
 			fi->unread = si->unread;
 			fi->total = si->total;
 			fi->flags = si->flags;
@@ -753,15 +726,16 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 static gboolean
 mapi_forget_folder (CamelMapiStore *mapi_store, const gchar *folder_name, GError **error)
 {
-	CamelMapiStorePrivate *priv = mapi_store->priv;
+	CamelService *service;
+	const gchar *user_data_dir;
 	gchar *state_file;
 	gchar *folder_dir, *storage_path;
 	CamelFolderInfo *fi;
-	const gchar *name;
 
-	name = folder_name;
+	service = CAMEL_SERVICE (mapi_store);
+	user_data_dir = camel_service_get_user_data_dir (service);
 
-	storage_path = g_strdup_printf ("%s/folders", priv->storage_path);
+	storage_path = g_strdup_printf ("%s/folders", user_data_dir);
 
 	/* Fixme Path - e_*-to_path */
 	folder_dir = g_strconcat (storage_path, "/", folder_name, NULL);
@@ -862,7 +836,6 @@ mapi_store_finalize (GObject *object)
 	priv = CAMEL_MAPI_STORE (object)->priv;
 
 	g_free (priv->profile);
-	g_free (priv->storage_path);
 	g_free (priv->base_url);
 
 	if (priv->id_hash != NULL)
@@ -904,12 +877,18 @@ mapi_store_can_refresh_folder (CamelStore *store,
                                CamelFolderInfo *info,
                                GError **error)
 {
+	CamelService *service;
+	CamelURL *url;
+
 	/* skip unselectable folders from automatic refresh */
 	if (info && (info->flags & CAMEL_FOLDER_NOSELECT) != 0)
 		return FALSE;
 
+	service = CAMEL_SERVICE (store);
+	url = camel_service_get_camel_url (service);
+
 	return CAMEL_STORE_CLASS(camel_mapi_store_parent_class)->can_refresh_folder (store, info, error) ||
-	      (camel_url_get_param (((CamelService *)store)->url, "check_all") != NULL);
+	      (camel_url_get_param (url, "check_all") != NULL);
 }
 
 static gboolean
@@ -936,9 +915,10 @@ mapi_store_get_folder_sync (CamelStore *store,
                             GError **error)
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-	CamelMapiStorePrivate *priv = mapi_store->priv;
+	CamelService *service;
 	CamelStoreInfo *si;
 	CamelFolder *folder;
+	const gchar *user_data_dir;
 	gchar *storage_path;
 
 	si = camel_mapi_store_summary_full_name (mapi_store->summary, folder_name);
@@ -969,7 +949,10 @@ mapi_store_get_folder_sync (CamelStore *store,
 	if (si)
 		camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, si);
 
-	storage_path = g_strdup_printf ("%s/folders", priv->storage_path);
+	service = CAMEL_SERVICE (store);
+	user_data_dir = camel_service_get_user_data_dir (service);
+
+	storage_path = g_strdup_printf ("%s/folders", user_data_dir);
 	folder = camel_mapi_folder_new (store, folder_name, storage_path, flags, error);
 	g_free (storage_path);
 
@@ -984,30 +967,34 @@ mapi_store_get_folder_info_sync (CamelStore *store,
                                  GError **error)
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
+	CamelServiceConnectionStatus status;
+	CamelService *service;
 
 	/*
 	 * Thanks to Michael, for his cached folders implementation in IMAP
 	 * is used as is here.
 	 */
 
-	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	service = CAMEL_SERVICE (store);
+	status = camel_service_get_connection_status (service);
+
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))) {
-		if (((CamelService *)store)->status == CAMEL_SERVICE_DISCONNECTED) {
-			((CamelService *)store)->status = CAMEL_SERVICE_CONNECTING;
-			mapi_connect_sync ((CamelService *)store, cancellable, NULL);
-		}
+		if (status == CAMEL_SERVICE_DISCONNECTED)
+			camel_service_connect_sync (service, NULL);
 
 		/* update folders from the server only when asking for the top most or the 'top' is not known;
 		   otherwise believe the local cache, because folders sync is pretty slow operation to be done
 		   one every single question on the folder info */
+		status = camel_service_get_connection_status (service);
 		if (((flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST) != 0 ||
 		    (!(flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED)) ||
 		    (!mapi_store->priv->folders_synced) ||
 		    (top && *top && !camel_mapi_store_folder_id_lookup (mapi_store, top))) &&
-		    (check_for_connection ((CamelService *)store, NULL) || ((CamelService *)store)->status == CAMEL_SERVICE_CONNECTING)) {
+		    (check_for_connection (service, NULL) || status == CAMEL_SERVICE_CONNECTING)) {
 			if (!mapi_folders_sync (mapi_store, flags, error)) {
-				camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+				camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 				return NULL;
 			}
 			camel_store_summary_touch ((CamelStoreSummary *)mapi_store->summary);
@@ -1015,7 +1002,7 @@ mapi_store_get_folder_info_sync (CamelStore *store,
 		}
 	}
 
-	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return mapi_get_folder_info_offline (store, top, flags, error);
 }
@@ -1189,16 +1176,21 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
 	CamelMapiStorePrivate  *priv = mapi_store->priv;
 	CamelStoreInfo *si = NULL;
+	CamelService *service;
+	const gchar *user_data_dir;
 	gchar *old_parent, *new_parent, *tmp;
 	gboolean move_cache = TRUE;
 	const gchar *old_fid_str, *new_parent_fid_str = NULL;
 	mapi_id_t old_fid;
 	GError *local_error = NULL;
 
-	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	service = CAMEL_SERVICE (store);
+	user_data_dir = camel_service_get_user_data_dir (service);
+
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (!camel_mapi_store_connected ((CamelMapiStore *)store, &local_error)) {
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 		if (local_error != NULL) {
 			g_propagate_error (error, local_error);
@@ -1216,7 +1208,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename MAPI folder '%s'. Folder does not exist."),
 			old_name);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return FALSE;
 	}
 
@@ -1228,7 +1220,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename MAPI default folder '%s' to '%s'."),
 			old_name, new_name);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return FALSE;
 	}
 
@@ -1255,7 +1247,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot rename MAPI folder '%s' to '%s'"),
 			old_name, new_name);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		g_free (old_parent);
 		g_free (new_parent);
 		return FALSE;
@@ -1282,7 +1274,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 					old_name, new_name);
 			}
 
-			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 			g_free (old_parent);
 			g_free (new_parent);
 			return FALSE;
@@ -1330,7 +1322,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 			   !exchange_mapi_util_mapi_id_from_string (old_parent_fid_str, &old_parent_fid) ||
 			   !exchange_mapi_util_mapi_id_from_string (new_parent_fid_str, &new_parent_fid) ||
 			   !exchange_mapi_connection_move_folder (priv->conn, old_fid, old_parent_fid, 0, new_parent_fid, 0, tmp, &local_error)) {
-			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+			camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 			if (local_error) {
 				g_set_error (
 					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
@@ -1364,7 +1356,7 @@ mapi_store_rename_folder_sync (CamelStore *store,
 		g_free (folder_id);
 	}
 
-	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	si = camel_store_summary_path ((CamelStoreSummary *)mapi_store->summary, old_name);
 	if (si) {
@@ -1380,8 +1372,8 @@ mapi_store_rename_folder_sync (CamelStore *store,
 	if (move_cache) {
 		gchar *oldpath, *newpath;
 
-		oldpath = g_build_filename (priv->storage_path, "folders", old_name, NULL);
-		newpath = g_build_filename (priv->storage_path, "folders", new_name, NULL);
+		oldpath = g_build_filename (user_data_dir, "folders", old_name, NULL);
+		newpath = g_build_filename (user_data_dir, "folders", new_name, NULL);
 
 		if (g_file_test (oldpath, G_FILE_TEST_IS_DIR) && g_rename (oldpath, newpath) == -1 && errno != ENOENT) {
 			g_warning ("Could not rename message cache '%s' to '%s': %s: cache reset", oldpath, newpath, g_strerror (errno));
@@ -1407,7 +1399,6 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si = NULL;
 	const gchar *parent_name = NULL, *use_folder_name = folder_name, *fid = NULL;
-	gboolean favourites = FALSE;
 	/* TODO : exchange_mapi_add_to_favorites (); */
 
 	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
@@ -1424,7 +1415,6 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 		}
 
 		use_folder_name = ++f_name;
-		favourites = TRUE;
 	}
 
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
@@ -1445,15 +1435,6 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 
 	if (si->flags & CAMEL_MAPI_FOLDER_MAIL) {
 		fi = mapi_build_folder_info (mapi_store, parent_name, use_folder_name);
-		if (favourites) {
-			CamelURL *url;
-			url = camel_url_new (mapi_store->priv->base_url, NULL);
-			url->path = g_strdup_printf ("/%s", camel_store_info_path (mapi_store->summary, si));
-			g_free (fi->uri);
-			fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
-			camel_url_free (url);
-		}
-
 		fi->unread = si->unread;
 		fi->total = si->total;
 		fi->flags = si->flags;
@@ -1463,8 +1444,11 @@ mapi_store_subscribe_folder_sync (CamelStore *store,
 		camel_store_folder_subscribed (store, fi);
 		camel_folder_info_free (fi);
 	} else {
+		CamelURL *url;
 		guint folder_type = mapi_folders_hash_table_type_lookup (mapi_store, folder_name);
-		exchange_mapi_add_esource (CAMEL_SERVICE(mapi_store)->url, use_folder_name, fid, folder_type);
+
+		url = camel_service_get_camel_url (CAMEL_SERVICE (mapi_store));
+		exchange_mapi_add_esource (url, use_folder_name, fid, folder_type);
 	}
 	camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
 	return TRUE;
@@ -1483,7 +1467,9 @@ mapi_store_unsubscribe_folder_sync (CamelStore *store,
 	gchar *f_name = NULL;
 
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-	CamelURL *url = CAMEL_SERVICE (mapi_store)->url;
+	CamelURL *url;
+
+	url = camel_service_get_camel_url (CAMEL_SERVICE (mapi_store));
 	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
 	if (si) {
@@ -1540,9 +1526,9 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	object_class = G_OBJECT_CLASS (class);
 	object_class->dispose = mapi_store_dispose;
 	object_class->finalize = mapi_store_finalize;
+	object_class->constructed = mapi_store_constructed;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
-	service_class->construct = mapi_construct;
 	service_class->get_name = mapi_get_name;
 	service_class->connect_sync = mapi_connect_sync;
 	service_class->disconnect_sync = mapi_disconnect_sync;
@@ -1578,26 +1564,27 @@ camel_mapi_store_init (CamelMapiStore *mapi_store)
 }
 
 /* service methods */
-static gboolean mapi_construct(CamelService *service, CamelSession *session,
-				 CamelProvider *provider, CamelURL *url,
-				 GError **error)
+static void
+mapi_store_constructed (GObject *object)
 {
-	CamelMapiStore	*mapi_store = CAMEL_MAPI_STORE (service);
-	CamelStore *store = CAMEL_STORE (service);
+	CamelMapiStore	*mapi_store = CAMEL_MAPI_STORE (object);
+	CamelStore *store = CAMEL_STORE (object);
 	CamelMapiStorePrivate *priv = mapi_store->priv;
+	CamelService *service;
+	CamelURL *url;
+	const gchar *user_data_dir;
 	gchar *path = NULL;
 
-	if (!CAMEL_SERVICE_CLASS (camel_mapi_store_parent_class)->construct (service, session, provider, url, error))
-		return FALSE;
+	/* Chain up to parent's constructed() method. */
+	G_OBJECT_CLASS (camel_mapi_store_parent_class)->constructed (object);
 
-	/*storage path*/
-	priv->storage_path = camel_session_get_storage_path (session, service, error);
-	if (!priv->storage_path)
-		return FALSE;
+	service = CAMEL_SERVICE (object);
+	url = camel_service_get_camel_url (service);
+	user_data_dir = camel_service_get_user_data_dir (service);
 
 	/*store summary*/
-	path = g_alloca (strlen (priv->storage_path) + 32);
-	sprintf (path, "%s/.summary", priv->storage_path);
+	path = g_alloca (strlen (user_data_dir) + 32);
+	sprintf (path, "%s/.summary", user_data_dir);
 
 	mapi_store->summary = camel_mapi_store_summary_new ();
 	camel_store_summary_set_filename ((CamelStoreSummary *)mapi_store->summary, path);
@@ -1609,9 +1596,11 @@ static gboolean mapi_construct(CamelService *service, CamelSession *session,
 	priv->profile = g_strdup (camel_url_get_param(url, "profile"));
 
 	/*base url*/
-	priv->base_url = camel_url_to_string (service->url, (CAMEL_URL_HIDE_PASSWORD |
-						       CAMEL_URL_HIDE_PARAMS   |
-						       CAMEL_URL_HIDE_AUTH)  );
+	priv->base_url = camel_url_to_string (
+		camel_service_get_camel_url (service),
+		CAMEL_URL_HIDE_PASSWORD |
+		CAMEL_URL_HIDE_PARAMS |
+		CAMEL_URL_HIDE_AUTH);
 
 	/*filter*/
 	if (camel_url_get_param (url, "filter"))
@@ -1627,21 +1616,23 @@ static gboolean mapi_construct(CamelService *service, CamelSession *session,
 	store->flags &= ~CAMEL_STORE_VTRASH;
 
 	store->flags |= CAMEL_STORE_SUBSCRIPTIONS | CAMEL_STORE_REAL_JUNK_FOLDER;
-
-	return TRUE;
 }
 
-static char
-*mapi_get_name(CamelService *service, gboolean brief)
+static char *
+mapi_get_name(CamelService *service, gboolean brief)
 {
+	CamelURL *url;
+
+	url = camel_service_get_camel_url (service);
+
 	if (brief) {
 		/* Translators: The %s is replaced with a server's host name */
-		return g_strdup_printf(_("Exchange MAPI server %s"), service->url->host);
+		return g_strdup_printf(_("Exchange MAPI server %s"), url->host);
 	} else {
 		/*To translators : Example string : Exchange MAPI service for
 		  _username_ on _server host name__*/
 		return g_strdup_printf(_("Exchange MAPI service for %s on %s"),
-				       service->url->user, service->url->host);
+				       url->user, url->host);
 	}
 }
 
@@ -1649,13 +1640,17 @@ static gboolean
 mapi_auth_loop (CamelService *service, GError **error)
 {
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
-	CamelSession *session = camel_service_get_session (service);
+	CamelSession *session;
+	CamelURL *url;
 
 	gchar *errbuf = NULL;
 	gboolean authenticated = FALSE;
 	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
 
-	service->url->passwd = NULL;
+	url = camel_service_get_camel_url (service);
+	session = camel_service_get_session (service);
+
+	url->passwd = NULL;
 
 	while (!authenticated) {
 		GError *mapi_error = NULL;
@@ -1663,11 +1658,11 @@ mapi_auth_loop (CamelService *service, GError **error)
 		if (errbuf) {
 			/* We need to un-cache the password before prompting again */
 			prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-			g_free (service->url->passwd);
-			service->url->passwd = NULL;
+			g_free (url->passwd);
+			url->passwd = NULL;
 		}
 
-		if (!service->url->passwd ) {
+		if (!url->passwd ) {
 			gchar *prompt;
 
 			/*To translators : First %s : is the error text or the reason
@@ -1676,16 +1671,16 @@ mapi_auth_loop (CamelService *service, GError **error)
 			 Third %s is : Server host name.*/
 			prompt = g_strdup_printf (_("%s Please enter the MAPI password for %s@%s"),
 						  errbuf ? errbuf : "",
-						  service->url->user,
-						  service->url->host);
-			service->url->passwd =
+						  url->user,
+						  url->host);
+			url->passwd =
 				camel_session_get_password (session, service, E_PASSWORD_COMPONENT,
 							    prompt, "password", prompt_flags, NULL);
 			g_free (prompt);
 			g_free (errbuf);
 			errbuf = NULL;
 
-			if (!service->url->passwd) {
+			if (!url->passwd) {
 				g_set_error (
 					error, G_IO_ERROR,
 					G_IO_ERROR_CANCELLED,
@@ -1694,7 +1689,7 @@ mapi_auth_loop (CamelService *service, GError **error)
 			}
 		}
 
-		store->priv->conn = exchange_mapi_connection_new (store->priv->profile, service->url->passwd, &mapi_error);
+		store->priv->conn = exchange_mapi_connection_new (store->priv->profile, url->passwd, &mapi_error);
 		if (!store->priv->conn || !exchange_mapi_connection_connected (store->priv->conn)) {
 			if (mapi_error) {
 				errbuf = g_strdup_printf (_("Unable to authenticate to Exchange MAPI server: %s"), mapi_error->message);
@@ -1715,20 +1710,14 @@ mapi_connect_sync (CamelService *service,
                    GError **error)
 {
 	CamelMapiStore *store = CAMEL_MAPI_STORE (service);
-	CamelMapiStorePrivate *priv = store->priv;
+	CamelServiceConnectionStatus status;
 	guint16 event_mask = 0;
 
-	if (service->status == CAMEL_SERVICE_DISCONNECTED) {
+	status = camel_service_get_connection_status (service);
+	if (status == CAMEL_SERVICE_DISCONNECTED) {
 		return FALSE;
 	}
 
-	if (!priv) {
-		store->priv = g_new0 (CamelMapiStorePrivate, 1);
-		priv = store->priv;
-		if (!camel_service_construct (service, service->session, service->provider, service->url, error))
-			return FALSE;
-	}
-
 	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	if (check_for_connection (service, NULL)) {
 		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -1741,7 +1730,6 @@ mapi_connect_sync (CamelService *service,
 		return FALSE;
 	}
 
-	service->status = CAMEL_SERVICE_CONNECTED;
 	camel_offline_store_set_online_sync (
 		CAMEL_OFFLINE_STORE (store), TRUE, cancellable, NULL);
 
@@ -1790,7 +1778,6 @@ mapi_disconnect_sync (CamelService *service,
 	}
 
 	store->priv->folders_synced = FALSE;
-	service->status = CAMEL_SERVICE_DISCONNECTED;
 
 	return TRUE;
 }
@@ -1912,10 +1899,8 @@ camel_mapi_store_summary_path_to_full(CamelMapiStoreSummary *s, const gchar *pat
 static CamelFolderInfo *
 mapi_build_folder_info(CamelMapiStore *mapi_store, const gchar *parent_name, const gchar *folder_name)
 {
-	CamelURL *url;
 	const gchar *name;
 	CamelFolderInfo *fi;
-	CamelMapiStorePrivate *priv = mapi_store->priv;
 
 	fi = camel_folder_info_new ();
 
@@ -1927,20 +1912,13 @@ mapi_build_folder_info(CamelMapiStore *mapi_store, const gchar *parent_name, con
 	else
 		fi->full_name = g_strdup (folder_name);
 
-	url = camel_url_new (priv->base_url, NULL);
-	g_free(url->path);
-
-	url->path = g_strdup_printf("/%s", fi->full_name);
-	fi->uri = camel_url_to_string(url,CAMEL_URL_HIDE_ALL);
-	camel_url_free(url);
-
 	name = strrchr(fi->full_name,'/');
 	if (name == NULL)
 		name = fi->full_name;
 	else
 		name++;
 
-	fi->name = g_strdup (name);
+	fi->display_name = g_strdup (name);
 
 	return fi;
 }
